#!/usr/bin/perl

#Class-based weighted fair queue interrogation module.
#This module maps class names to policy-id, object-id pairs and
#does an SNMP get on the bytes transmitted OID.
#written by Andy Brauer
#initial release 07/05/2002
#current release 26/04/2005
#current version 0.53
#
# 25/04/2005 - Paul Wozney
#
# Completely rewritten by Paul Wozney
# There was nothing really wrong with the script that Andy wrote, 
# but I extended it to respond to index, query and get requests
# and made seperate subroutines for these
# 
# I did this so I could integrate the script into cacti, as this
# was the only script available - and I really couldn't follow
# the code that Andy wrote so I figured it was easier just to 
# do it my way.  :)
#
# I'm sure there are bugs.  Reach me here: paulwozney@gmail.com
#
# One caveat, as I'm currently applying DSCP and policing QoS on
# the same router I named my QoS classes like this "QoSnnnn" and
# this script is written to see those.
#
# April 29 2005 - I have changed the code so it can handle
# both purely numeric, and text based snmpwalk output

#use warnings;
use strict;

my $ip = $ARGV[0];
my $community = $ARGV[1];
my $action1 = $ARGV[2];
my $action2 = $ARGV[3];
my $class = $ARGV[4];
my $filterstring = "QoS";	# use this to filter out unwanted classes
#my $filterstring = ".*";	# or to match everything 

my $snmppath = "/usr/bin/snmpwalk";
my $version = "2c";
my $index_oid = "1.3.6.1.4.1.9.9.166.1.7.1.1.1";	# lists the classes
my $ilist_oid = "1.3.6.1.4.1.9.9.166.1.5.1.1.2";	# lists the references
my $bytes_oid = "1.3.6.1.4.1.9.9.166.1.15.1.1.9";	# lists the pol/obj id refs

sub retrieve_index {

# lists the classes
# this is used in the other subroutines, so if a third variable is passed "index"
# it lists the references along with the classes

	my $node = shift;
	my $comm = shift;
	my $ind = shift;
        my @classes;

	my @index = `$snmppath -v $version $node -c $comm $index_oid`;
	# should output this:
	# SNMPv2-SMI::enterprises.9.9.166.1.7.1.1.1.1025 = STRING: "CLASS"

        foreach (@index) {
                s/"//g;
                my @littlearray = split / /;
		# if you've set $filterstring it will only display those classes you want
		next unless ($littlearray[3] =~ /$filterstring/);
		my @values = split /\./, $littlearray[0];
		if ($ind eq "index") { push @classes, "$values[-1]:$littlearray[3]\n" ; }
                else { push @classes, "$values[-1]\n"; }
        }
	return @classes;
}

sub retrieve_query {

# retrieves the bytes for all classes

	my $node = shift;
	my $comm = shift;
	my $dowhatever = shift; # the data to retrieve
	my @classes = @_;	# gets the remaining variables passed
	my @bytes;

	my @ilist = `$snmppath -v $version $node -c $comm $ilist_oid`;
	# should output this:
	# SNMPv2-SMI::enterprises.9.9.166.1.5.1.1.2.1073.1073 = Gauge32: 1025
	# note that the variable part matches the oid for the class above

	foreach my $entry (@ilist) {
		my @littlearray = split / /, $entry;
		my @values = split /\./, $littlearray[0];
		chomp $littlearray[3];
		foreach (@classes) {
			my ($refer, $class) = split /:/;
			chomp $class;
			chomp $refer;
			# if you've set $filterstring it will only display those classes you want
			if (($littlearray[3] eq $refer) && ($class =~ /$filterstring/)) {
				my $byte_count = `$snmppath -v $version $node -c $comm $bytes_oid.$values[-2].$values[-1]` ;
				my @byte_array = split / /, $byte_count;
				if ($dowhatever eq "class") { push @bytes, "$refer:$class"; }
				if ($dowhatever eq "bytes") { push @bytes, "$refer:$byte_array[3]"; }
			}
		}
	}
	return @bytes;
}

sub retrieve_get {

# gets the bytes for a single class
# this executes in about 10 seconds on my box, it could be 
# way faster if there was some kind of caching done, maybe 
# expire after an hour considering most people poll every 5 mins
#
# NOTE!  I changed some internals and now it does sub-second
# execution! Woot!

	my $node = shift;
	my $comm = shift;
	my $colour = shift;	# just another variable for class, but 
				# I hate scope problems.  :)
	chomp $colour;
	my @classes = @_;
	my $indexval;

	my @ilist = `$snmppath -v $version $node -c $comm $ilist_oid`;

	# then figure out the oid
	foreach my $entry (@ilist) {
		my @littlearray = split / /, $entry;
		my @values = split /\./, $littlearray[0];
		chomp $littlearray[3];
		if ($colour == $littlearray[3]) { $indexval = "$values[-2].$values[-1]" ; }
	}

	# and POW!  it's done.
	my $byte_count = `$snmppath -v $version $node -c $comm $bytes_oid.$indexval`;
	my @byte_array = split / /, $byte_count;

	return "$byte_array[3]";
}

ARGUMENTS: {
	if ($action1 eq "index") { print retrieve_index($ip,$community) ; last ARGUMENTS; }
	if ($action1 eq "query") { 
		if ($action2 eq "bytes") {
			print retrieve_query($ip,$community,"bytes",retrieve_index($ip,$community,"index")) ; 
			last ARGUMENTS; 
		}
		elsif ($action2 eq "class") {
			print retrieve_query($ip,$community,"class",retrieve_index($ip,$community,"index")) ; 
			last ARGUMENTS; 
		}
	}
	if ($action1 eq "get") { print retrieve_get($ip,$community,$class,retrieve_index($ip,$community,"index")) ; last ARGUMENTS; }
	print "usage:\n\n./nmiscbwfq.pl IP COMMUNITY index\n./nmiscbwfq.pl IP COMMUNITY query {bytes|class}\n./nmiscbwfq.pl IP COMMUNITY get {bytes} CLASS\n";
}

exit;
